=begin
#The Plaid API

#The Plaid REST API. Please see https://plaid.com/docs/api for more details.

The version of the OpenAPI document: 2020-09-14_1.586.4

Generated by: https://openapi-generator.tech
OpenAPI Generator version: 6.3.0

=end

require 'date'
require 'time'

module Plaid
  # Defines the request schema for `/transfer/authorization/create`
  class TransferAuthorizationCreateRequest
    # Your Plaid API `client_id`. The `client_id` is required and may be provided either in the `PLAID-CLIENT-ID` header or as part of a request body.
    attr_accessor :client_id

    # Your Plaid API `secret`. The `secret` is required and may be provided either in the `PLAID-SECRET` header or as part of a request body.
    attr_accessor :secret

    # The Plaid `access_token` for the account that will be debited or credited.
    attr_accessor :access_token

    # The Plaid `account_id` corresponding to the end-user account that will be debited or credited.
    attr_accessor :account_id

    # Specify the account used to fund the transfer. Should be specified if using legacy funding methods only. If using Plaid Ledger, leave this field blank. Customers can find a list of `funding_account_id`s in the Accounts page of your Plaid Dashboard, under the \"Account ID\" column. If this field is left blank and you are using legacy funding methods, this will default to the default `funding_account_id` specified during onboarding. Otherwise, Plaid Ledger will be used.
    attr_accessor :funding_account_id

    # Specify which ledger balance used to fund the transfer. Customers can find a list of `ledger_id`s in the Accounts page of your Plaid Dashboard. If this field is left blank, this will default to id of the default ledger balance.
    attr_accessor :ledger_id

    # The payment profile token associated with the Payment Profile that will be debited or credited. Required if not using `access_token`.
    attr_accessor :payment_profile_token

    attr_accessor :type

    attr_accessor :network

    # The amount of the transfer (decimal string with two digits of precision e.g. \"10.00\"). When calling `/transfer/authorization/create`, specify the maximum amount to authorize. When calling `/transfer/create`, specify the exact amount of the transfer, up to a maximum of the amount authorized. If this field is left blank when calling `/transfer/create`, the maximum amount authorized in the `authorization_id` will be sent.
    attr_accessor :amount

    attr_accessor :ach_class

    attr_accessor :wire_details

    attr_accessor :user

    attr_accessor :device

    # Plaid's unique identifier for the origination account for this authorization. If not specified, the default account will be used.
    attr_accessor :origination_account_id

    # The currency of the transfer amount. The default value is \"USD\".
    attr_accessor :iso_currency_code

    # A random key provided by the client, per unique authorization, which expires after 48 hours. Maximum of 50 characters.  The API supports idempotency for safely retrying requests without accidentally performing the same operation twice. For example, if a request to create an authorization fails due to a network connection error, you can retry the request with the same idempotency key to guarantee that only a single authorization is created.  Idempotency does not apply to authorizations whose decisions are `user_action_required`. Therefore you may re-attempt the authorization after completing the required user action without changing `idempotency_key`.  This idempotency key expires after 48 hours, after which the same key can be reused. Failure to provide this key may result in duplicate charges.
    attr_accessor :idempotency_key

    # If the end user is initiating the specific transfer themselves via an interactive UI, this should be `true`; for automatic recurring payments where the end user is not actually initiating each individual transfer, it should be `false`. This field is not currently used and is present to support planned future functionality.
    attr_accessor :user_present

    # If set to `false`, Plaid will not offer a `guarantee_decision` for this request (Guarantee customers only).
    attr_accessor :with_guarantee

    # The unique identifier returned by Plaid's [beacon](https://plaid.com/docs/transfer/guarantee/#using-a-beacon) when it is run on your webpage.
    attr_accessor :beacon_session_id

    # The Plaid client ID that is the originator of this transfer. Only needed if creating transfers on behalf of another client as a [Platform customer](https://plaid.com/docs/transfer/application/#originators-vs-platforms).
    attr_accessor :originator_client_id

    attr_accessor :credit_funds_source

    # Plaid’s unique identifier for a test clock. This field may only be used when using `sandbox` environment. If provided, the `authorization` is created at the `virtual_time` on the provided test clock.
    attr_accessor :test_clock_id

    class EnumAttributeValidator
      attr_reader :datatype
      attr_reader :allowable_values

      def initialize(datatype, allowable_values)
        @allowable_values = allowable_values.map do |value|
          case datatype.to_s
          when /Integer/i
            value.to_i
          when /Float/i
            value.to_f
          else
            value
          end
        end
      end

      def valid?(value)
        !value || allowable_values.include?(value)
      end
    end

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        :'client_id' => :'client_id',
        :'secret' => :'secret',
        :'access_token' => :'access_token',
        :'account_id' => :'account_id',
        :'funding_account_id' => :'funding_account_id',
        :'ledger_id' => :'ledger_id',
        :'payment_profile_token' => :'payment_profile_token',
        :'type' => :'type',
        :'network' => :'network',
        :'amount' => :'amount',
        :'ach_class' => :'ach_class',
        :'wire_details' => :'wire_details',
        :'user' => :'user',
        :'device' => :'device',
        :'origination_account_id' => :'origination_account_id',
        :'iso_currency_code' => :'iso_currency_code',
        :'idempotency_key' => :'idempotency_key',
        :'user_present' => :'user_present',
        :'with_guarantee' => :'with_guarantee',
        :'beacon_session_id' => :'beacon_session_id',
        :'originator_client_id' => :'originator_client_id',
        :'credit_funds_source' => :'credit_funds_source',
        :'test_clock_id' => :'test_clock_id'
      }
    end

    # Returns all the JSON keys this model knows about
    def self.acceptable_attributes
      attribute_map.values
    end

    # Attribute type mapping.
    def self.openapi_types
      {
        :'client_id' => :'String',
        :'secret' => :'String',
        :'access_token' => :'String',
        :'account_id' => :'String',
        :'funding_account_id' => :'String',
        :'ledger_id' => :'String',
        :'payment_profile_token' => :'String',
        :'type' => :'TransferType',
        :'network' => :'TransferNetwork',
        :'amount' => :'String',
        :'ach_class' => :'ACHClass',
        :'wire_details' => :'TransferWireDetails',
        :'user' => :'TransferAuthorizationUserInRequest',
        :'device' => :'TransferAuthorizationDevice',
        :'origination_account_id' => :'String',
        :'iso_currency_code' => :'String',
        :'idempotency_key' => :'String',
        :'user_present' => :'Boolean',
        :'with_guarantee' => :'Boolean',
        :'beacon_session_id' => :'String',
        :'originator_client_id' => :'String',
        :'credit_funds_source' => :'TransferCreditFundsSource',
        :'test_clock_id' => :'String'
      }
    end

    # List of attributes with nullable: true
    def self.openapi_nullable
      Set.new([
        :'funding_account_id',
        :'ledger_id',
        :'wire_details',
        :'idempotency_key',
        :'user_present',
        :'with_guarantee',
        :'beacon_session_id',
        :'originator_client_id',
        :'test_clock_id'
      ])
    end

    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    def initialize(attributes = {})
      if (!attributes.is_a?(Hash))
        fail ArgumentError, "The input argument (attributes) must be a hash in `Plaid::TransferAuthorizationCreateRequest` initialize method"
      end

      # check to see if the attribute exists and convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h|
        if (!self.class.attribute_map.key?(k.to_sym))
          fail ArgumentError, "`#{k}` is not a valid attribute in `Plaid::TransferAuthorizationCreateRequest`. Please check the name to make sure it's valid. List of attributes: " + self.class.attribute_map.keys.inspect
        end
        h[k.to_sym] = v
      }

      if attributes.key?(:'client_id')
        self.client_id = attributes[:'client_id']
      end

      if attributes.key?(:'secret')
        self.secret = attributes[:'secret']
      end

      if attributes.key?(:'access_token')
        self.access_token = attributes[:'access_token']
      end

      if attributes.key?(:'account_id')
        self.account_id = attributes[:'account_id']
      end

      if attributes.key?(:'funding_account_id')
        self.funding_account_id = attributes[:'funding_account_id']
      end

      if attributes.key?(:'ledger_id')
        self.ledger_id = attributes[:'ledger_id']
      end

      if attributes.key?(:'payment_profile_token')
        self.payment_profile_token = attributes[:'payment_profile_token']
      end

      if attributes.key?(:'type')
        self.type = attributes[:'type']
      end

      if attributes.key?(:'network')
        self.network = attributes[:'network']
      end

      if attributes.key?(:'amount')
        self.amount = attributes[:'amount']
      end

      if attributes.key?(:'ach_class')
        self.ach_class = attributes[:'ach_class']
      end

      if attributes.key?(:'wire_details')
        self.wire_details = attributes[:'wire_details']
      end

      if attributes.key?(:'user')
        self.user = attributes[:'user']
      end

      if attributes.key?(:'device')
        self.device = attributes[:'device']
      end

      if attributes.key?(:'origination_account_id')
        self.origination_account_id = attributes[:'origination_account_id']
      end

      if attributes.key?(:'iso_currency_code')
        self.iso_currency_code = attributes[:'iso_currency_code']
      end

      if attributes.key?(:'idempotency_key')
        self.idempotency_key = attributes[:'idempotency_key']
      end

      if attributes.key?(:'user_present')
        self.user_present = attributes[:'user_present']
      end

      if attributes.key?(:'with_guarantee')
        self.with_guarantee = attributes[:'with_guarantee']
      else
        self.with_guarantee = true
      end

      if attributes.key?(:'beacon_session_id')
        self.beacon_session_id = attributes[:'beacon_session_id']
      end

      if attributes.key?(:'originator_client_id')
        self.originator_client_id = attributes[:'originator_client_id']
      end

      if attributes.key?(:'credit_funds_source')
        self.credit_funds_source = attributes[:'credit_funds_source']
      end

      if attributes.key?(:'test_clock_id')
        self.test_clock_id = attributes[:'test_clock_id']
      end
    end

    # Show invalid properties with the reasons. Usually used together with valid?
    # @return Array for valid properties with the reasons
    def list_invalid_properties
      invalid_properties = Array.new
      if @access_token.nil?
        invalid_properties.push('invalid value for "access_token", access_token cannot be nil.')
      end

      if @account_id.nil?
        invalid_properties.push('invalid value for "account_id", account_id cannot be nil.')
      end

      if @type.nil?
        invalid_properties.push('invalid value for "type", type cannot be nil.')
      end

      if @network.nil?
        invalid_properties.push('invalid value for "network", network cannot be nil.')
      end

      if @amount.nil?
        invalid_properties.push('invalid value for "amount", amount cannot be nil.')
      end

      if @user.nil?
        invalid_properties.push('invalid value for "user", user cannot be nil.')
      end

      if !@idempotency_key.nil? && @idempotency_key.to_s.length > 50
        invalid_properties.push('invalid value for "idempotency_key", the character length must be smaller than or equal to 50.')
      end


      invalid_properties
    end

    # Check to see if the all the properties in the model are valid
    # @return true if the model is valid
    def valid?
      return false if @access_token.nil?
      return false if @account_id.nil?
      return false if @type.nil?
      return false if @network.nil?
      return false if @amount.nil?
      return false if @user.nil?
      return false if !@idempotency_key.nil? && @idempotency_key.to_s.length > 50
      true
    end

    # Custom attribute writer method with validation
    # @param [Object] idempotency_key Value to be assigned
    def idempotency_key=(idempotency_key)
      if !idempotency_key.nil? && idempotency_key.to_s.length > 50
        fail ArgumentError, 'invalid value for "idempotency_key", the character length must be smaller than or equal to 50.'
      end


      @idempotency_key = idempotency_key
    end

    # Checks equality by comparing each attribute.
    # @param [Object] Object to be compared
    def ==(o)
      return true if self.equal?(o)
      self.class == o.class &&
          client_id == o.client_id &&
          secret == o.secret &&
          access_token == o.access_token &&
          account_id == o.account_id &&
          funding_account_id == o.funding_account_id &&
          ledger_id == o.ledger_id &&
          payment_profile_token == o.payment_profile_token &&
          type == o.type &&
          network == o.network &&
          amount == o.amount &&
          ach_class == o.ach_class &&
          wire_details == o.wire_details &&
          user == o.user &&
          device == o.device &&
          origination_account_id == o.origination_account_id &&
          iso_currency_code == o.iso_currency_code &&
          idempotency_key == o.idempotency_key &&
          user_present == o.user_present &&
          with_guarantee == o.with_guarantee &&
          beacon_session_id == o.beacon_session_id &&
          originator_client_id == o.originator_client_id &&
          credit_funds_source == o.credit_funds_source &&
          test_clock_id == o.test_clock_id
    end

    # @see the `==` method
    # @param [Object] Object to be compared
    def eql?(o)
      self == o
    end

    # Calculates hash code according to all attributes.
    # @return [Integer] Hash code
    def hash
      [client_id, secret, access_token, account_id, funding_account_id, ledger_id, payment_profile_token, type, network, amount, ach_class, wire_details, user, device, origination_account_id, iso_currency_code, idempotency_key, user_present, with_guarantee, beacon_session_id, originator_client_id, credit_funds_source, test_clock_id].hash
    end

    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def self.build_from_hash(attributes)
      new.build_from_hash(attributes)
    end

    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)
      attributes = attributes.transform_keys(&:to_sym)
      self.class.openapi_types.each_pair do |key, type|
        if attributes[self.class.attribute_map[key]].nil? && self.class.openapi_nullable.include?(key)
          self.send("#{key}=", nil)
        elsif type =~ /\AArray<(.*)>/i
          # check to ensure the input is an array given that the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            self.send("#{key}=", attributes[self.class.attribute_map[key]].map { |v| _deserialize($1, v) })
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          self.send("#{key}=", _deserialize(type, attributes[self.class.attribute_map[key]]))
        end
      end

      self
    end

    # Deserializes the data based on type
    # @param string type Data type
    # @param string value Value to be deserialized
    # @return [Object] Deserialized data
    def _deserialize(type, value)
      case type.to_sym
      when :Time
        Time.parse(value)
      when :Date
        Date.parse(value)
      when :String
        value.to_s
      when :Integer
        value.to_i
      when :Float
        value.to_f
      when :Boolean
        if value.to_s =~ /\A(true|t|yes|y|1)\z/i
          true
        else
          false
        end
      when :Object
        # generic object (usually a Hash), return directly
        value
      when /\AArray<(?<inner_type>.+)>\z/
        inner_type = Regexp.last_match[:inner_type]
        value.map { |v| _deserialize(inner_type, v) }
      when /\AHash<(?<k_type>.+?), (?<v_type>.+)>\z/
        k_type = Regexp.last_match[:k_type]
        v_type = Regexp.last_match[:v_type]
        {}.tap do |hash|
          value.each do |k, v|
            hash[_deserialize(k_type, k)] = _deserialize(v_type, v)
          end
        end
      else # model
        # models (e.g. Pet) or oneOf
        klass = Plaid.const_get(type)
        klass.respond_to?(:openapi_one_of) ? klass.build(value) : klass.build_from_hash(value)
      end
    end

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # to_body is an alias to to_hash (backward compatibility)
    # @return [Hash] Returns the object in the form of hash
    def to_body
      to_hash
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = self.send(attr)
        if value.nil?
          is_nullable = self.class.openapi_nullable.include?(attr)
          next if !is_nullable || (is_nullable && !instance_variable_defined?(:"@#{attr}"))
        end

        hash[param] = _to_hash(value)
      end
      hash
    end

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end

  end

end
