=begin
#The Plaid API

#The Plaid REST API. Please see https://plaid.com/docs/api for more details.

The version of the OpenAPI document: 2020-09-14_1.586.4

Generated by: https://openapi-generator.tech
OpenAPI Generator version: 6.3.0

=end

require 'date'
require 'time'

module Plaid
  # Represents a transfer intent within Transfer UI.
  class TransferIntentGet
    # Plaid's unique identifier for a transfer intent object.
    attr_accessor :id

    # The datetime the transfer was created. This will be of the form `2006-01-02T15:04:05Z`.
    attr_accessor :created

    attr_accessor :status

    # Plaid's unique identifier for the transfer created through the UI. Returned only if the transfer was successfully created. Null value otherwise.
    attr_accessor :transfer_id

    attr_accessor :failure_reason

    attr_accessor :authorization_decision

    attr_accessor :authorization_decision_rationale

    # The Plaid `account_id` for the account that will be debited or credited. Returned only if `account_id` was set on intent creation.
    attr_accessor :account_id

    # Plaid’s unique identifier for the origination account used for the transfer.
    attr_accessor :origination_account_id

    # The id of the funding account to use, available in the Plaid Dashboard. This determines which of your business checking accounts will be credited or debited.
    attr_accessor :funding_account_id

    # The amount of the transfer (decimal string with two digits of precision e.g. \"10.00\"). When calling `/transfer/authorization/create`, specify the maximum amount to authorize. When calling `/transfer/create`, specify the exact amount of the transfer, up to a maximum of the amount authorized. If this field is left blank when calling `/transfer/create`, the maximum amount authorized in the `authorization_id` will be sent.
    attr_accessor :amount

    attr_accessor :mode

    attr_accessor :network

    attr_accessor :ach_class

    attr_accessor :user

    # A description for the underlying transfer. Maximum of 8 characters.
    attr_accessor :description

    # The Metadata object is a mapping of client-provided string fields to any string value. The following limitations apply: The JSON values must be Strings (no nested JSON objects allowed) Only ASCII characters may be used Maximum of 50 key/value pairs Maximum key length of 40 characters Maximum value length of 500 characters 
    attr_accessor :metadata

    # The currency of the transfer amount, e.g. \"USD\"
    attr_accessor :iso_currency_code

    # When `true`, the transfer requires a `GUARANTEED` decision by Plaid to proceed (Guarantee customers only).
    attr_accessor :require_guarantee

    attr_accessor :guarantee_decision

    attr_accessor :guarantee_decision_rationale

    class EnumAttributeValidator
      attr_reader :datatype
      attr_reader :allowable_values

      def initialize(datatype, allowable_values)
        @allowable_values = allowable_values.map do |value|
          case datatype.to_s
          when /Integer/i
            value.to_i
          when /Float/i
            value.to_f
          else
            value
          end
        end
      end

      def valid?(value)
        !value || allowable_values.include?(value)
      end
    end

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        :'id' => :'id',
        :'created' => :'created',
        :'status' => :'status',
        :'transfer_id' => :'transfer_id',
        :'failure_reason' => :'failure_reason',
        :'authorization_decision' => :'authorization_decision',
        :'authorization_decision_rationale' => :'authorization_decision_rationale',
        :'account_id' => :'account_id',
        :'origination_account_id' => :'origination_account_id',
        :'funding_account_id' => :'funding_account_id',
        :'amount' => :'amount',
        :'mode' => :'mode',
        :'network' => :'network',
        :'ach_class' => :'ach_class',
        :'user' => :'user',
        :'description' => :'description',
        :'metadata' => :'metadata',
        :'iso_currency_code' => :'iso_currency_code',
        :'require_guarantee' => :'require_guarantee',
        :'guarantee_decision' => :'guarantee_decision',
        :'guarantee_decision_rationale' => :'guarantee_decision_rationale'
      }
    end

    # Returns all the JSON keys this model knows about
    def self.acceptable_attributes
      attribute_map.values
    end

    # Attribute type mapping.
    def self.openapi_types
      {
        :'id' => :'String',
        :'created' => :'Time',
        :'status' => :'TransferIntentStatus',
        :'transfer_id' => :'String',
        :'failure_reason' => :'TransferIntentGetFailureReason',
        :'authorization_decision' => :'TransferIntentAuthorizationDecision',
        :'authorization_decision_rationale' => :'TransferAuthorizationDecisionRationale',
        :'account_id' => :'String',
        :'origination_account_id' => :'String',
        :'funding_account_id' => :'String',
        :'amount' => :'String',
        :'mode' => :'TransferIntentCreateMode',
        :'network' => :'TransferIntentCreateNetwork',
        :'ach_class' => :'ACHClass',
        :'user' => :'TransferUserInResponse',
        :'description' => :'String',
        :'metadata' => :'Hash<String, String>',
        :'iso_currency_code' => :'String',
        :'require_guarantee' => :'Boolean',
        :'guarantee_decision' => :'TransferAuthorizationGuaranteeDecision',
        :'guarantee_decision_rationale' => :'TransferAuthorizationGuaranteeDecisionRationale'
      }
    end

    # List of attributes with nullable: true
    def self.openapi_nullable
      Set.new([
        :'transfer_id',
        :'failure_reason',
        :'authorization_decision',
        :'authorization_decision_rationale',
        :'account_id',
        :'metadata',
        :'require_guarantee',
        :'guarantee_decision',
        :'guarantee_decision_rationale'
      ])
    end

    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    def initialize(attributes = {})
      if (!attributes.is_a?(Hash))
        fail ArgumentError, "The input argument (attributes) must be a hash in `Plaid::TransferIntentGet` initialize method"
      end

      # check to see if the attribute exists and convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h|
        if (!self.class.attribute_map.key?(k.to_sym))
          fail ArgumentError, "`#{k}` is not a valid attribute in `Plaid::TransferIntentGet`. Please check the name to make sure it's valid. List of attributes: " + self.class.attribute_map.keys.inspect
        end
        h[k.to_sym] = v
      }

      if attributes.key?(:'id')
        self.id = attributes[:'id']
      end

      if attributes.key?(:'created')
        self.created = attributes[:'created']
      end

      if attributes.key?(:'status')
        self.status = attributes[:'status']
      end

      if attributes.key?(:'transfer_id')
        self.transfer_id = attributes[:'transfer_id']
      end

      if attributes.key?(:'failure_reason')
        self.failure_reason = attributes[:'failure_reason']
      end

      if attributes.key?(:'authorization_decision')
        self.authorization_decision = attributes[:'authorization_decision']
      end

      if attributes.key?(:'authorization_decision_rationale')
        self.authorization_decision_rationale = attributes[:'authorization_decision_rationale']
      end

      if attributes.key?(:'account_id')
        self.account_id = attributes[:'account_id']
      end

      if attributes.key?(:'origination_account_id')
        self.origination_account_id = attributes[:'origination_account_id']
      end

      if attributes.key?(:'funding_account_id')
        self.funding_account_id = attributes[:'funding_account_id']
      end

      if attributes.key?(:'amount')
        self.amount = attributes[:'amount']
      end

      if attributes.key?(:'mode')
        self.mode = attributes[:'mode']
      end

      if attributes.key?(:'network')
        self.network = attributes[:'network']
      else
        self.network = 'same-day-ach'
      end

      if attributes.key?(:'ach_class')
        self.ach_class = attributes[:'ach_class']
      end

      if attributes.key?(:'user')
        self.user = attributes[:'user']
      end

      if attributes.key?(:'description')
        self.description = attributes[:'description']
      end

      if attributes.key?(:'metadata')
        if (value = attributes[:'metadata']).is_a?(Hash)
          self.metadata = value
        end
      end

      if attributes.key?(:'iso_currency_code')
        self.iso_currency_code = attributes[:'iso_currency_code']
      end

      if attributes.key?(:'require_guarantee')
        self.require_guarantee = attributes[:'require_guarantee']
      end

      if attributes.key?(:'guarantee_decision')
        self.guarantee_decision = attributes[:'guarantee_decision']
      end

      if attributes.key?(:'guarantee_decision_rationale')
        self.guarantee_decision_rationale = attributes[:'guarantee_decision_rationale']
      end
    end

    # Show invalid properties with the reasons. Usually used together with valid?
    # @return Array for valid properties with the reasons
    def list_invalid_properties
      invalid_properties = Array.new
      if @id.nil?
        invalid_properties.push('invalid value for "id", id cannot be nil.')
      end

      if @created.nil?
        invalid_properties.push('invalid value for "created", created cannot be nil.')
      end

      if @status.nil?
        invalid_properties.push('invalid value for "status", status cannot be nil.')
      end

      if @origination_account_id.nil?
        invalid_properties.push('invalid value for "origination_account_id", origination_account_id cannot be nil.')
      end

      if @funding_account_id.nil?
        invalid_properties.push('invalid value for "funding_account_id", funding_account_id cannot be nil.')
      end

      if @amount.nil?
        invalid_properties.push('invalid value for "amount", amount cannot be nil.')
      end

      if @mode.nil?
        invalid_properties.push('invalid value for "mode", mode cannot be nil.')
      end

      if @user.nil?
        invalid_properties.push('invalid value for "user", user cannot be nil.')
      end

      if @description.nil?
        invalid_properties.push('invalid value for "description", description cannot be nil.')
      end


      if @iso_currency_code.nil?
        invalid_properties.push('invalid value for "iso_currency_code", iso_currency_code cannot be nil.')
      end

      invalid_properties
    end

    # Check to see if the all the properties in the model are valid
    # @return true if the model is valid
    def valid?
      return false if @id.nil?
      return false if @created.nil?
      return false if @status.nil?
      return false if @origination_account_id.nil?
      return false if @funding_account_id.nil?
      return false if @amount.nil?
      return false if @mode.nil?
      return false if @user.nil?
      return false if @description.nil?
      return false if !@metadata.nil? && @metadata.length > 50
      return false if @iso_currency_code.nil?
      true
    end

    # Custom attribute writer method with validation
    # @param [Object] metadata Value to be assigned
    def metadata=(metadata)

      @metadata = metadata
    end

    # Checks equality by comparing each attribute.
    # @param [Object] Object to be compared
    def ==(o)
      return true if self.equal?(o)
      self.class == o.class &&
          id == o.id &&
          created == o.created &&
          status == o.status &&
          transfer_id == o.transfer_id &&
          failure_reason == o.failure_reason &&
          authorization_decision == o.authorization_decision &&
          authorization_decision_rationale == o.authorization_decision_rationale &&
          account_id == o.account_id &&
          origination_account_id == o.origination_account_id &&
          funding_account_id == o.funding_account_id &&
          amount == o.amount &&
          mode == o.mode &&
          network == o.network &&
          ach_class == o.ach_class &&
          user == o.user &&
          description == o.description &&
          metadata == o.metadata &&
          iso_currency_code == o.iso_currency_code &&
          require_guarantee == o.require_guarantee &&
          guarantee_decision == o.guarantee_decision &&
          guarantee_decision_rationale == o.guarantee_decision_rationale
    end

    # @see the `==` method
    # @param [Object] Object to be compared
    def eql?(o)
      self == o
    end

    # Calculates hash code according to all attributes.
    # @return [Integer] Hash code
    def hash
      [id, created, status, transfer_id, failure_reason, authorization_decision, authorization_decision_rationale, account_id, origination_account_id, funding_account_id, amount, mode, network, ach_class, user, description, metadata, iso_currency_code, require_guarantee, guarantee_decision, guarantee_decision_rationale].hash
    end

    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def self.build_from_hash(attributes)
      new.build_from_hash(attributes)
    end

    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)
      attributes = attributes.transform_keys(&:to_sym)
      self.class.openapi_types.each_pair do |key, type|
        if attributes[self.class.attribute_map[key]].nil? && self.class.openapi_nullable.include?(key)
          self.send("#{key}=", nil)
        elsif type =~ /\AArray<(.*)>/i
          # check to ensure the input is an array given that the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            self.send("#{key}=", attributes[self.class.attribute_map[key]].map { |v| _deserialize($1, v) })
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          self.send("#{key}=", _deserialize(type, attributes[self.class.attribute_map[key]]))
        end
      end

      self
    end

    # Deserializes the data based on type
    # @param string type Data type
    # @param string value Value to be deserialized
    # @return [Object] Deserialized data
    def _deserialize(type, value)
      case type.to_sym
      when :Time
        Time.parse(value)
      when :Date
        Date.parse(value)
      when :String
        value.to_s
      when :Integer
        value.to_i
      when :Float
        value.to_f
      when :Boolean
        if value.to_s =~ /\A(true|t|yes|y|1)\z/i
          true
        else
          false
        end
      when :Object
        # generic object (usually a Hash), return directly
        value
      when /\AArray<(?<inner_type>.+)>\z/
        inner_type = Regexp.last_match[:inner_type]
        value.map { |v| _deserialize(inner_type, v) }
      when /\AHash<(?<k_type>.+?), (?<v_type>.+)>\z/
        k_type = Regexp.last_match[:k_type]
        v_type = Regexp.last_match[:v_type]
        {}.tap do |hash|
          value.each do |k, v|
            hash[_deserialize(k_type, k)] = _deserialize(v_type, v)
          end
        end
      else # model
        # models (e.g. Pet) or oneOf
        klass = Plaid.const_get(type)
        klass.respond_to?(:openapi_one_of) ? klass.build(value) : klass.build_from_hash(value)
      end
    end

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # to_body is an alias to to_hash (backward compatibility)
    # @return [Hash] Returns the object in the form of hash
    def to_body
      to_hash
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = self.send(attr)
        if value.nil?
          is_nullable = self.class.openapi_nullable.include?(attr)
          next if !is_nullable || (is_nullable && !instance_variable_defined?(:"@#{attr}"))
        end

        hash[param] = _to_hash(value)
      end
      hash
    end

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end

  end

end
